
CLIENT = {
	ONBOARDSIDE = {
		NONE = 0,
		LEFT = 1,
		RIGHT = 2,
		BACK = 3,
		FRONT = 4
	},
	ClassName = "CLIENT",
	ClientName = nil,
	ClientAlive = false,
	ClientTransport = false,
	_Menus = {},
	_Cargos = {},
	_Tasks = {},
	Messages = { 
	}
}

function CLIENT:New( ClientName, ClientBriefing )

  -- Arrange meta tables
  local Child = BASE:Inherit( self, BASE:New() )
  Child.ClientName = ClientName
  Child.AddBriefing( Child, ClientBriefing )
  return Child
end

function CLIENT:Reset( ClientName )
	self._Menus = {}
	self._Cargos = {}
end

function CLIENT:ClientGroup()
	return Group.getByName( self.ClientName )
end 

function CLIENT:Transport()
	env.info( "Client:Transport()" )

	self.ClientTransport = true
end

function CLIENT:AddBriefing( ClientBriefing )
	self.ClientBriefing = ClientBriefing
end

function CLIENT:IsTransport()
	return self.ClientTransport
end

function CLIENT:FindCargo( CargoName )
	return self._Cargos[CargoName]
end

function CLIENT:ShowCargo()
  --env.info( "Client:ShowCargo()" )

  local CargoMsg = ""
  
  for CargoName, Cargo in pairs( self._Cargos ) do
    if CargoMsg  ~= "" then
      CargoMsg = CargoMsg .. "\n"
    end
    CargoMsg = CargoMsg .. "Co-Pilot: Cargo: " .. Cargo.CargoName .. " Type:" ..  _TransportTypeText[Cargo.CargoType] .. " Weight: " .. Cargo.CargoWeight
  end
  
  if CargoMsg == '' then
	CargoMsg = "Co-Pilot: Cargo: empty"
  end
  
  self:Message( CargoMsg, 15, self.ClientName .. "/Cargo", 30 )

end

function CLIENT:AddCargo( CargoName, CargoGroupName, CargoType, CargoWeight, CargoGroupTemplate )
  env.info( "Client:AddCargo()" )

  local Valid = true
  
  Valid = _TransportValidateString( CargoName, "CargoName", Valid )
  Valid = _TransportValidateEnum( CargoType, "CargoType", _TransportType, Valid )
  Valid = _TransportValidateNumber( CargoWeight, "CargoWeight", Valid )

  if  Valid then
    local Cargo = {}
    Cargo.CargoName = CargoName
    Cargo.CargoGroupName = CargoGroupName
    Cargo.CargoType = CargoType
    Cargo.CargoWeight = CargoWeight
	Cargo.CargoGroupTemplate = CargoGroupTemplate
    self._Cargos[CargoName] = Cargo
    self:ShowCargo()
  end
  
  return Valid
  
end
    
function CLIENT:RemoveCargo( CargoName )
  env.info( "Client:RemoveCargo()" )

  local Valid = true
  local Cargo = nil
  
  Valid = _TransportValidateString( CargoName, "CargoName", Valid )

  if  Valid then
    env.info( "Client:RemoveCargo: CargoName = " .. CargoName )
    Cargo = mist.utils.deepCopy( self._Cargos[CargoName] )
    self._Cargos[CargoName] = nil
  end
  
  return Cargo
  
end

function CLIENT:Message( Message, MessageDuration, MessageId, MessageInterval )

	if self.Messages[MessageId] == nil then
		self.Messages[MessageId] = {}
		self.Messages[MessageId].MessageId = MessageId
		self.Messages[MessageId].MessageTime = timer.getTime()
		self.Messages[MessageId].MessageDuration = MessageDuration
		if MessageInterval == nil then
			self.Messages[MessageId].MessageInterval = 300
		else
			self.Messages[MessageId].MessageInterval = MessageInterval
		end
		MessageToGroup( self:ClientGroup(), Message, MessageDuration,  MessageId )
	else
		if self:ClientGroup() and self:ClientGroup():getUnits() and self:ClientGroup():getUnits()[1] and not self:ClientGroup():getUnits()[1]:inAir() then
			if timer.getTime() - self.Messages[MessageId].MessageTime  >= self.Messages[MessageId].MessageDuration - 1 then
				MessageToGroup( self:ClientGroup(), Message, MessageDuration,  MessageId )
				self.Messages[MessageId].MessageTime = timer.getTime()
			end
		else
			if timer.getTime() - self.Messages[MessageId].MessageTime  >= self.Messages[MessageId].MessageDuration + self.Messages[MessageId].MessageInterval then
				MessageToGroup( self:ClientGroup(), Message, MessageDuration,  MessageId )
				self.Messages[MessageId].MessageTime = timer.getTime()
			end
		end
	end
end
