--require "main"

--[[  
      Class MISSION
--]]

MISSION = {
  ClassName = "MISSION",
  Name = "",
  MissionStatus = "PENDING",
  _Clients = {},
  _Tasks = {},
  _Cargos = {},
  GoalFunction = nil,
  MissionReportTrigger = 0,
  MissionReportShow = false,
  MissionReportFlash = false,
  MissionTimeInterval = 0,
  MissionCoalition = "",
  SUCCESS = 1,
  FAILED = 2,
  REPEAT = 3
}

CARGOSTATUS = {
	NONE = 0,
	LOADED = 1,
	UNLOADED = 2,
	LOADING = 3,
	LoadCount= 0,
	UnloadCount = 0
}

function MISSION:Meta()
  env.info( "MISSION:Meta()" )

  -- Arrange meta tables
  local Child = BASE:Inherit( self, BASE:New() )
  return Child
end

function MISSION:New( MissionName, MissionPriority, MissionBriefing, MissionCoalition )
  env.info( "MISSION:New()" )
  Child = MISSION:Meta()
  
  local Valid = true
  
  Valid = _TransportValidateString( MissionName, "MissionName", Valid )
  Valid = _TransportValidateString( MissionPriority, "MissionPriority", Valid )
  Valid = _TransportValidateString( MissionBriefing, "MissionBriefing", Valid )
  Valid = _TransportValidateString( MissionCoalition, "MissionCoalition", Valid )
  
  if Valid then
    Child.Name = MissionName
	Child.MissionPriority = MissionPriority
	Child.MissionBriefing = MissionBriefing
	Child.MissionCoalition = MissionCoalition
  end

  return Child
  
end

function MISSION:Failed()

	self.MissionStatus = "FAILED"

end

function MISSION:IsFailed() 

	return self.MissionStatus == "FAILED"

end

function MISSION:StatusToClients()
	if self.MissionReportTrigger >= self.MissionTimeInterval then
		for ClientID, Client in pairs( self._Clients ) do
			Client:Message( self.MissionCoalition .. ' command: Mission status "' .. self.Name .. '": ' .. Mission.MissionStatus .. '! ( ' .. self.MissionPriority .. ' mission ) ', 10,  self.Name .. '/Status')
		end
	end
end

function MISSION:ReportTrigger()
	if self.MissionReportShow == true then
		self.MissionReportShow = false
		return true
	else
		if self.MissionReportFlash == true then
			self.MissionReportTrigger = self.MissionReportTrigger + 1
			if self.MissionReportTrigger >= self.MissionTimeInterval then
				self.MissionReportTrigger = 0
				return true
			else
				return false
			end
		else
			return false 
		end
	end
end

function MISSION:ReportToAll()
	local AlivePlayers = ''
	for ClientID, Client in pairs( self._Clients ) do
		if  Client:ClientGroup() then
			if Client:ClientGroup():getUnits() and Client:ClientGroup():getUnits()[1] then
				if Client:ClientGroup():getUnits()[1]:getLife() > 0.0 then
					if AlivePlayers == '' then
						AlivePlayers = ' Players: ' .. Client:ClientGroup():getUnits()[1]:getPlayerName()
					else
						AlivePlayers = AlivePlayers .. ' / ' .. Client:ClientGroup():getUnits()[1]:getPlayerName()
					end
				end
			end
		end
	end
	MessageToAll( self.MissionCoalition .. ' command: Mission report "' .. self.Name .. '": ' .. self.MissionStatus .. ' ( ' .. self.MissionPriority .. ' mission )' .. AlivePlayers, 10,  self.Name .. '/Status')
end

function MISSION:Ongoing()
	env.info('MISSION:Ongoing()' )
	self.MissionStatus = "ONGOING"
	self:StatusToClients()
end

function MISSION:Pending()
	env.info('MISSION:Pending()' )
	self.MissionStatus = "PENDING"
	self:StatusToClients()
end

function MISSION:Completed()
	env.info('MISSION:Completed()' )
	self.MissionStatus = "ACCOMPLISHED"
	self:StatusToClients()
end

function MISSION:IsCompleted()
	return self.MissionStatus == "ACCOMPLISHED" 
end

function MISSION:AddGoal( GoalFunction )
	self.GoalFunction = GoalFunction 

end

function MISSION:ShowBriefing( Client )
	env.info( "MISSION:ShowBriefing: " .. Client.ClientName )

	local Valid = true
 
	if Valid then
		Client:Message( self.MissionCoalition .. ' command: Mission briefing "' .. self.Name .. '": ' .. self.MissionBriefing, 30,  self.Name .. '/MissionBriefing' )
		if Client.ClientBriefing then
			Client:Message( self.MissionCoalition .. ' command: Your briefing: ' .. Client.ClientBriefing, 30,  self.Name .. '/ClientBriefing' )
		end
	end

	return Client
end


function MISSION:AddClient( Client )
  env.info( "MISSION:AddClient = " .. Client.ClientName )

  local Valid = true
 
  if Valid then
    self._Clients[Client.ClientName] = Client
  end

  return Client

end

function MISSION:FindClient( ClientName )
	return self._Clients[ClientName]
end


function MISSION:AddTask( Task, TaskNumber )

  local Valid = true 

  if Valid then
    self._Tasks[TaskNumber] = Task
  end

  return Task
  
end

function MISSION:AddCargo( CargoName, CargoType, CargoWeight, CargoGroupControlCenter, CargoGroupTemplate, CargoZone )

	local Cargo = {}
	Cargo.CargoName = CargoName
	if CargoType == _TransportType.INFANTRY then
		if CargoGroupControlCenter == nil then
			Cargo.CargoGroupName = SpawnInfantryZone( CargoZone, CargoGroupTemplate )
		else
			Cargo.CargoGroupName = SpawnInfantry( Group.getByName( CargoGroupControlCenter ), CargoGroupTemplate, CargoZone, 0, 0, nil, false )
			trigger.action.activateGroup( Group.getByName( Cargo.CargoGroupName ) )
			--trigger.action.setGroupAIOff( Cargo.CargoGroupName )

		end
    else
		Cargo.CargoGroupName = CargoGroupControlCenter
	end
	Cargo.CargoType = CargoType
    Cargo.CargoWeight = CargoWeight
	Cargo.CargoGroupControlCenter = CargoGroupControlCenter
	Cargo.CargoGroupTemplate = CargoGroupTemplate
	Cargo.CargoZone = CargoZone
	Cargo.Status = CARGOSTATUS.NONE
    self._Cargos[CargoName] = Cargo

	return Cargo.CargoGroupName
	
end


--[[
  _TransportType: Defines the different types of transports, which has an impact on the menu commands shown in F10.
  
  - _TransportType.TROOPS
  - _TransportType.GOODS
  - _TransportType.VEHICLES
  - _TransportType.CARGO
--]]
_TransportType = { 
  TROOPS    = 1, 
  GOODS     = 2, 
  VEHICLES  = 3, 
  INFANTRY  = 4,
  PATIENTS  = 5,
  PACKAGE   = 6,
  CARGO     = 7
}

_TransportTypeText = {
  "Troops", 
  "Goods",
  "Vehicles",
  "Infantry",
  "Patients",
  "Package",
  "Cargo"
}


--[[
  _TransportExecuteStage: Defines the different stages of Transport unload/load execution. This table is internal and is used to control the validity of Transport load/unload timing.
  
  - _TransportExecuteStage.EXECUTING
  - _TransportExecuteStage.SUCCESS
  - _TransportExecuteStage.FAILED
  
--]]
_TransportExecuteStage = { 
  NONE = 0,
  EXECUTING = 1, 
  SUCCESS = 2, 
  FAILED = 3
}




function _TransportValidateUnitInZone( TransportUnit, TransportZone )
  --env.info( "_TransportValidateUnitInZone()" )

  local Valid = true

  -- fill-up some local variables to support further calculations to determine location of units within the zone.
  local TransportUnitPos = TransportUnit:getPosition().p
  if TransportZone then
    TransportZonePos = {radius = TransportZone.radius, x = TransportZone.point.x, y = TransportZone.point.y, z = TransportZone.point.z}
  end

  if  ((( TransportUnitPos.x - TransportZonePos.x)^2 + (TransportUnitPos.z - TransportZonePos.z)^2)^0.5 <= TransportZonePos.radius) then
  else
    Valid = false
  end
  
  return Valid
end

function _TransportValidateUnitInRadius( CargoUnit, ReferenceGroup, Radius )
  --env.info( "_TransportValidateUnitInRadius()" )

  local Valid = true

  -- fill-up some local variables to support further calculations to determine location of units within the zone.
  local CargoPos = CargoUnit:getPosition().p
  local ReferenceGroupPos = ReferenceGroup:getUnits()[1]:getPosition().p

  if  (((CargoPos.x - ReferenceGroupPos.x)^2 + (CargoPos.z - ReferenceGroupPos.z)^2)^0.5 <= Radius) then
  else
    Valid = false
  end
  
  return Valid
end

function _TransportValidateGroupUnitsInRadius( CargoGroup, ReferenceGroup, Radius )
  --env.info( "_TransportValidateGroupUnitsInRadius()" )

  local Valid = true
  
  Valid = _TransportValidateGroup( CargoGroup, "CargoGroup", Valid )
  Valid = _TransportValidateGroup( ReferenceGroup, "ReferenceGroup", Valid )
  
  -- fill-up some local variables to support further calculations to determine location of units within the zone
  local CargoUnits = CargoGroup:getUnits()
  for CargoUnitId, CargoUnit in pairs( CargoUnits ) do
    local CargoUnitPos = CargoUnit:getPosition().p
--    env.info( '_TransportValidateGroupUnitsInRadius: CargoUnitPos.x = ' .. CargoUnitPos.x .. ' CargoUnitPos.z = ' .. CargoUnitPos.z )
    local ReferenceGroupPos = ReferenceGroup:getUnits()[1]:getPosition().p
--    env.info( '_TransportValidateGroupUnitsInRadius: ReferenceGroupPos.x = ' .. ReferenceGroupPos.x .. ' ReferenceGroupPos.z = ' .. ReferenceGroupPos.z )

    if  ((( CargoUnitPos.x - ReferenceGroupPos.x)^2 + (CargoUnitPos.z - ReferenceGroupPos.z)^2)^0.5 <= Radius) then
    else
      Valid = false
      break
    end
  end
  
  return Valid
end

function _TransportDestroyGroupUnitsInRadius( CargoGroup, ReferenceGroup, Radius )
  --env.info( "_TransportDestroyGroupInRadius()" )

  local Valid = true
  
  Valid = _TransportValidateGroup( CargoGroup, "CargoGroup", Valid )
  Valid = _TransportValidateGroup( ReferenceGroup, "ReferenceGroup", Valid )
  
  if Valid then
    -- fill-up some local variables to support further calculations to determine location of units within the zone
    local CargoUnits = CargoGroup:getUnits()
    local AliveCargoUnits = #CargoUnits
    for CargoUnitId, CargoUnit in pairs( CargoUnits ) do
      local CargoUnitPos = CargoUnit:getPosition().p
--      env.info( '_TransportDestroyGroupUnitsInRadius: CargoUnitPos.x = ' .. CargoUnitPos.x .. ' CargoUnitPos.z = ' .. CargoUnitPos.z )
      local ReferenceGroupPos = ReferenceGroup:getUnits()[1]:getPosition().p
--      env.info( '_TransportDestroyGroupUnitsInRadius: ReferenceGroupPos.x = ' .. ReferenceGroupPos.x .. ' ReferenceGroupPos.z = ' .. ReferenceGroupPos.z )

      if  ((( CargoUnitPos.x - ReferenceGroupPos.x)^2 + (CargoUnitPos.z - ReferenceGroupPos.z)^2)^0.5 <= Radius) then
        CargoUnit:destroy()
        AliveCargoUnits = AliveCargoUnits - 1
      else
        Valid = false
        break
      end
    end
  else
    AliveCargoUnits = -1
  end
  
  return AliveCargoUnits
end


function _TransportValidateString( Variable, VariableName, Valid )

  if  type( Variable ) == "string" then
    if Variable == "" then
      error( "_TransportValidateString: error: " .. VariableName .. " must be filled out!" )
      Valid = false
    end
  else
    error( "_TransportValidateString: error: " .. VariableName .. " is not a string." )
    Valid = false
  end

  return Valid
  
end

function _TransportValidateNumber( Variable, VariableName, Valid )

  if  type( Variable ) == "number" then
  else
    error( "_TransportValidateNumber: error: " .. VariableName .. " is not a number." )
    Valid = false
  end

  return Valid
  
end

function _TransportValidateGroup( Variable, VariableName, Valid )

	if Variable == nil then
		error( "_TransportValidateGroup: error: " .. VariableName .. " is a nil value!" )
		Valid = false
	end

	return Valid
  
end

function _TransportValidateZone( Variable, VariableName, Valid )

	if Variable == nil then
		error( "_TransportValidateGroup: error: " .. VariableName .. " is a nil value!" )
		Valid = false
	end
  
	if trigger.misc.getZone( Variable ) == nil then 
		error( "_TransportValidateGroup: error: Zone " .. VariableName .. " does not exist!" )
		Valid = false
	end

	return Valid
  
end

function _TransportValidateEnum( Variable, VariableName, Enum, Valid )

  local ValidVariable = false
  
  for EnumId, EnumData in pairs( Enum ) do
    if Variable == EnumData then
      ValidVariable = true
      break
    end
  end
  
  if  ValidVariable then
  else
    error( 'TransportValidateEnum: " .. VariableName .. " is not a valid type.' .. Variable )
    Valid = false
  end

  return Valid
  
end


MISSIONSCHEDULER = {
  Missions = {},
  MissionCount = 0,
  TimeIntervalCount = 0,
  TimeIntervalShow = 150,
  TimeSeconds = 14400,
  TimeShow = 5
}

function MISSIONSCHEDULER.Scheduler()

	env.info( "MISSIONSCHEDULER.Scheduler(): Begin" )
	-- loop through the missions in the TransportTasks
	for MissionName, Mission in pairs( MISSIONSCHEDULER.Missions ) do
		
		if not Mission:IsCompleted() then
			
			for ClientID, Client in pairs( Mission._Clients ) do
				--local Client = Mission._Clients[1]

				local ClientDead = false
				
				if  Client:ClientGroup() then
					if Client:ClientGroup():getUnits() and Client:ClientGroup():getUnits()[1] then
						if Client:ClientGroup():getUnits()[1]:getLife() > 0.0 then

							if not Client.ClientAlive then
								Client._Tasks = mist.utils.deepCopy( Mission._Tasks )
								Client.ClientAlive = true
								Mission:Ongoing()
								if Mission.GoalFunction ~= nil then
									MissionComplete =  Mission.GoalFunction( Mission, Client )
								end						
							end
							
							-- for each mission, check for each TransportMissionTask the state
							MissionComplete = true 

							for TaskNumber, Task in pairs( Client._Tasks ) do

								if not Task.Stage then
									Task:SetStage( 1 )
								end

								env.info( 'Scheduler: Mission.Name = ' .. Mission.Name .. ' / Task.Name = ' .. Task.Name .. ' / Task.ActiveStage = ' .. Task.ActiveStage .. ' / Task.Stage.Name = ' .. Task.Stage.Name .. ' / Client "' .. Client:ClientGroup():getName() .. '"' )
								
								local TransportTime = timer.getTime()
						
								if not Task:IsDone() then
									-- Action
									if Task:StageExecute() then
										Task.Stage:Execute( Mission, Client, Task )
									end
								  
									-- Wait until execution is finished            
									if  Task.ExecuteStage == _TransportExecuteStage.EXECUTING then
										Task.Stage:Executing( Mission, Client, Task )
									end
								  
									-- Validate completion or reverse to earlier stage
									if Task.Time + Task.Stage.WaitTime <= TransportTime  then
										Task:SetStage( Task.Stage:Validate( Mission, Client, Task ) )
									end

									if Task:IsDone() then
										env.info( 'Scheduler: Mission '.. Mission.Name .. ' Task ' .. Task.Name .. ' Stage ' .. Task.Stage.Name .. ' done. MissionComplete = ' .. string.format ( "%s", MissionComplete and "true" or "false" ) )
									else
										-- break only if this task is not yet done, so that future task are not yet activated.
										MissionComplete = false -- when a task is not yet completed, a mission cannot be completed
										--env.info( 'Scheduler: Mission "'.. Mission.Name .. '" Task "' .. Task.Name .. '" Stage "' .. Task.Stage.Name .. '" break. MissionComplete = ' .. string.format ( "%s", MissionComplete and "true" or "false" ) )
										break
									end
								end
							end
							
							if MissionComplete then
								if Mission.GoalFunction ~= nil then
									MissionGoal = Mission.GoalFunction( Mission, Client )
								end
								if MissionGoal == MISSION.SUCCESS then
									Mission:Completed()
								else
									if MissionGoal == MISSION.FAILED then
										Mission:Failed()
									else
										-- reset the task queue and start again
										Client._Tasks = mist.utils.deepCopy( Mission._Tasks )
									end
								end
							end

						else
							if Client.ClientAlive then
								env.info( 'Scheduler: Client "' .. Client.ClientName .. '" is dead.' )
								Client.ClientAlive = false
								Mission:Pending()
							end
						end
					else
						if Client.ClientAlive then
							env.info( 'Scheduler: Client "' .. Client.ClientName .. '" has no units.' )
							Client.ClientAlive = false
							Mission:Pending()
						end
					end
				else
					if Client.ClientAlive then
						env.info( 'Scheduler: Client "' .. Client.ClientName .. '" is inactive.' )
						Client.ClientAlive = false
						Mission:Pending()
					end
					--env.info( 'Scheduler: Client "' .. Client.ClientName .. '" does not exist.' )
				end

			end

			Mission:StatusToClients()
		else
			Mission:StatusToClients()
		end
		
		if Mission:ReportTrigger() then
			Mission:ReportToAll()
		end
		
	end

	env.info( "MISSIONSCHEDULER.Scheduler(): End" )
	
end

function MISSIONSCHEDULER.Start()
  if MISSIONSCHEDULER ~= nil then
    MISSIONSCHEDULER.SchedulerId = mist.scheduleFunction( MISSIONSCHEDULER.Scheduler, { }, 0, 2 )
  end
end

function MISSIONSCHEDULER.Stop()
  if MISSIONSCHEDULER.SchedulerId then
    mist.removeFunction(MISSIONSCHEDULER.SchedulerId)
    MISSIONSCHEDULER.SchedulerId = nil
  end
end

function MISSIONSCHEDULER.AddMission( MissionName, MissionPriority, MissionBriefing, MissionCoalition )
  local MissionAdd = MISSION:New( MissionName, MissionPriority, MissionBriefing, MissionCoalition )
  MISSIONSCHEDULER.Missions[MissionName] = MissionAdd
  MISSIONSCHEDULER.MissionCount = MISSIONSCHEDULER.MissionCount + 1
  return MissionAdd
end

function MISSIONSCHEDULER.RemoveMission( MissionName )
	env.info( 'MISSIONSCHEDULER.RemoveMission( ' .. MissionName .. ' )' )
	MISSIONSCHEDULER.Missions[MissionName] = nil
  MISSIONSCHEDULER.MissionCount = MISSIONSCHEDULER.MissionCount - 1
end

function MISSIONSCHEDULER.FindMission( MissionName )
  return MISSIONSCHEDULER.Missions[MissionName]
end

function MISSIONSCHEDULER.ReportMissionsShow( )
	for MissionName, Mission in pairs( MISSIONSCHEDULER.Missions ) do
		Mission.MissionReportShow = true
		Mission.MissionReportFlash = false 
	end
end

function MISSIONSCHEDULER.ReportMissionsFlash( TimeInterval )
	env.info( 'MISSIONSCHEDULER.ReportMissionsFlash( ' .. TimeInterval .. ')' )
	local Count = 0
	for MissionName, Mission in pairs( MISSIONSCHEDULER.Missions ) do
		Mission.MissionReportShow = false 
		Mission.MissionReportFlash = true
		Mission.MissionReportTrigger = Count * TimeInterval
		Mission.MissionTimeInterval = MISSIONSCHEDULER.MissionCount * TimeInterval 
		Count = Count + 1
	end
end

function MISSIONSCHEDULER.ReportMissionsHide( Prm )
	for MissionName, Mission in pairs( MISSIONSCHEDULER.Missions ) do
		Mission.MissionReportShow = false
		Mission.MissionReportFlash = false
	end
end

function MISSIONSCHEDULER.ReportMenu()
	local ReportMenu = SUBMENU:New( 'Status' )
	local ReportMenuShow = COMMANDMENU:New( 'Show Report Missions', ReportMenu, MISSIONSCHEDULER.ReportMissionsShow, 0 )
	local ReportMenuFlash = COMMANDMENU:New('Flash Report Missions', ReportMenu, MISSIONSCHEDULER.ReportMissionsFlash, 10 )
	local ReportMenuHide = COMMANDMENU:New( 'Hide Report Missions', ReportMenu, MISSIONSCHEDULER.ReportMissionsHide, 0 )
end

function MISSIONSCHEDULER:TimeShow()
	self.TimeIntervalCount = self.TimeIntervalCount + 1
	if self.TimeIntervalCount >= self.TimeTriggerShow then
		local TimeMsg = string.format("%00d", ( self.TimeSeconds / 60 ) - ( timer.getTime() / 60 )) .. ' minutes left until mission reload.'
		MessageToAll( TimeMsg, self.TimeShow, '/TimeMsg' )
		self.TimeIntervalCount = 0
	end

end

function MISSIONSCHEDULER:Time( TimeSeconds, TimeIntervalShow, TimeShow )

	self.TimeIntervalCount = 0
	self.TimeSeconds = TimeSeconds
	self.TimeIntervalShow = TimeIntervalShow
	self.TimeShow = TimeShow

end


--[[

-- Test Constructors

local Mission = MISSIONSCHEDULER.AddMission( 'Deploy Troops' )

local Client = CLIENT:New( Group )
Client:AddCargo( 'Infantry', 'RU MI-8MTV2 - Infantry AI', _TransportType.INFANTRY, 4000 )

Mission:AddClient( Client )

local DeployTask = DEPLOYTASK:New( 'RU Test Zone', 'Infantry', _TransportType.INFANTRY )

DeployTask:SetCargoTargetZoneName( 'RU Troops Attack 5' )
DeployTask:AddSmokeRed( 'RU Smoke Test' )

Mission:AddTask( DeployTask, 1 )

local PickupTask = PICKUPTASK:New( 'RU Test Zone', 'Infantry2', 'RU Infantry', _TransportType.INFANTRY, 115, CLIENT.ONBOARDSIDE.LEFT )
PickupTask:AddSmokeGreen( 'RU Smoke Test' )
Mission:AddTask( PickupTask, 2 )


--]]

