
STAGE = {
  ClassName = "STAGE",
  MSG = { ID = "None", TIME = 10 },
  FREQUENCY = { NONE = 0, ONCE = 1, REPEAT = -1 },
  
  Name = "NoStage",
  WaitTime = 1,
  Frequency = 1,
  MessageCount = 0,
  MessageInterval = 15,
  MessageShown = {},
  MessageShow = false,
  MessageFlash = false
}


function STAGE:New()
  --env.info( "STAGE:New()" )
  local Child = BASE:Inherit( self, BASE:New() )
  return Child
end

function STAGE:Execute( Mission, Client, Task )

	local Valid = true
  
	return Valid
end

function STAGE:Executing( Mission, Client, Task )

end

function STAGE:Validate( Mission, Client, Task )

  local Valid = true

  return Valid
end



STAGEBRIEF = {
	ClassName = "BRIEF",
	MSG = { ID = "Brief", TIME = 30 },
	Name = "Brief",
	StageBriefingTime = 0,
	StageBriefingDuration = 10
}

function STAGEBRIEF:New()
	-- Arrange meta tables
	local Child = BASE:Inherit( self, STAGE:New() )
	return Child
end

function STAGEBRIEF:Execute( Mission, Client, Task )
	local Valid = BASE:Inherited(self):Execute( Mission, Client, Task )
	Mission:ShowBriefing( Client )
	self.StageBriefingTime = timer.getTime()
	return Valid 
end

function STAGEBRIEF:Validate( Mission, Client, Task )
	local Valid = STAGE:Validate( Mission, Client, Task )

	if timer.getTime() - self.StageBriefingTime <= self.StageBriefingDuration then
		return 0
	else
		self.StageBriefingTime = timer.getTime()
		return 1
	end
  
end


STAGESTART = {
  ClassName = "START",
  MSG = { ID = "Start", TIME = 30 },
  Name = "Start",
  StageStartTime = 0,
  StageStartDuration = 10
}

function STAGESTART:New()
	-- Arrange meta tables
	local Child = BASE:Inherit( self, STAGE:New() )
	return Child
end

function STAGESTART:Execute( Mission, Client, Task )
	local Valid = BASE:Inherited(self):Execute( Mission, Client, Task )
	if Task.TaskBriefing then
		Client:Message( Mission.MissionCoalition .. ' command: ' .. Task.TaskBriefing, self.StageStartDuration,  Mission.Name .. "/Stage" )
	else
		Client:Message( Mission.MissionCoalition .. ' command: Task: Task ' .. Task.TaskNumber .. '.', self.StageStartDuration, Mission.Name .. "/Stage" )
	end
	self.StageStartTime = timer.getTime()
	return Valid 
end

function STAGESTART:Validate( Mission, Client, Task )
	local Valid = STAGE:Validate( Mission, Client, Task )

	if timer.getTime() - self.StageStartTime <= self.StageStartDuration then
		return 0
	else
		self.StageStartTime = timer.getTime()
		return 1
	end
  
	return 1
  
end

STAGEROUTE = {
  ClassName = "STAGEROUTE",
  MSG = { ID = "Route", TIME = 5 },
  Frequency = STAGE.FREQUENCY.REPEAT,
  Name = "Route"
}

function STAGEROUTE:New()
  -- Arrange meta tables
  local Child = BASE:Inherit( self, STAGE:New() )
  return Child
end

function STAGEROUTE:Execute( Mission, Client, Task )
	local Valid = BASE:Inherited(self):Execute( Mission, Client, Task )
	SendRouteMessageToCarrier( Client:ClientGroup(), Task.LandingZoneName, self.MSG.TIME, Mission.Name .. "/Stage" )
	if Client:IsTransport() then
		Client:ShowCargo()
	end
	return Valid
end

function STAGEROUTE:Validate( Mission, Client, Task )
  local Valid = STAGE:Validate( Mission, Client, Task )

  -- check if this carrier is in the landing zone
  if  _TransportValidateUnitInZone( Client:ClientGroup():getUnits()[1], trigger.misc.getZone(Task.LandingZoneName) ) then
    return 1
  end
  
  return 0
  
end

STAGELANDING = {
  ClassName = "STAGELANDING",
  MSG = { ID = "Landing", TIME = 10 },
  Name = "Landing",
  Signalled = false
}

function STAGELANDING:New()
  -- Arrange meta tables
  local Child = BASE:Inherit( self, STAGE:New() )
  return Child
end

function STAGELANDING:Execute( Mission, Client, Task )
 
	Client:Message( 'Co-Pilot: We have arrived at ' .. Task.LandingZoneName .. '. Land the helicopter to ' .. Task.TEXT[1] .. ' the ' .. _TransportTypeText[Task.CargoType] .. '.', self.MSG.TIME,  Mission.Name .. "/Stage" )
  
    if not self.Signalled then
	  
		if Task.LandingZoneSignalType then
			env.info( 'TransportSchedule: Task.LandingZoneSignalType = ' .. Task.LandingZoneSignalType.TEXT )
			if Task.LandingZoneSignalVehicle then
				env.info( 'TransportSchedule: Task.LandingZoneSignalVehicle = ' .. Task.LandingZoneSignalVehicle )
		  
				local SignalGroup = Group.getByName(Task.LandingZoneSignalVehicle)
				local SignalUnit
				if SignalGroup == nil then
					SignalUnit = StaticObject.getByName( Task.LandingZoneSignalVehicle )
				else
					SignalUnit = SignalGroup:getUnits()[1]
				end
				if SignalUnit ~= nil then
					env.info( 'TransportSchedule: Signalling Unit' )
					local SignalVehiclePos = SignalUnit:getPosition().p
					SignalVehiclePos.y = SignalVehiclePos.y + Task.LandingZoneSignalHeight
					if  	Task.LandingZoneSignalType.ID == Task.SIGNAL.TYPE.SMOKE.ID then
						trigger.action.smoke( SignalVehiclePos, Task.LandingZoneSignalColor.COLOR )
					elseif  Task.LandingZoneSignalType.ID == Task.SIGNAL.TYPE.FLARE.ID then
						trigger.action.signalFlare( SignalVehiclePos, Task.LandingZoneSignalColor.COLOR, 0 )
					end
				end
			else
				env.info( 'TransportSchedule: Signaling landing zone ' )
		  
				local LandingZone = TransportZone
				local CurrentPosition = { x = LandingZone.point.x, y = LandingZone.point.z }
				LandingZone.point.y = land.getHeight( CurrentPosition ) + 10
		  
				if    Task.LandingZoneSignalType.ID == Task.SIGNAL.TYPE.SMOKE.ID then
					env.info( 'TransportSchedule: Smoking zone x = ' .. LandingZone.point.x .. ' y = ' .. LandingZone.point.y .. ' z = ' .. LandingZone.point.z )
					trigger.action.smoke( TransportZone.point, Task.LandingZoneSignalColor.COLOR )
				elseif  Task.LandingZoneSignalType.ID == Task.SIGNAL.TYPE.SMOKE.FLARE.ID then
					env.info( 'TransportSchedule: Flaring zone x = ' .. LandingZone.point.x .. ' y = ' .. LandingZone.point.y .. ' z = ' .. LandingZone.point.z )
					trigger.action.signalFlare( TransportZone.point, Task.LandingZoneSignalColor.COLOR, 0 )
				end
			end
		end
		self.Signalled = true 
	end
end

function STAGELANDING:Validate( Mission, Client, Task )
  
  if  _TransportValidateUnitInZone( Client:ClientGroup():getUnits()[1], trigger.misc.getZone(Task.LandingZoneName) ) then
  else
    return -1
  end
  
  if not Client:ClientGroup():getUnits()[1]:inAir() then
  else
    return 0
  end
  
  return 1
end

STAGELANDED = {
  ClassName = "STAGELANDED",
  MSG = { ID = "Land", TIME = 10 },
  Name = "Landed",
  MenusAdded = false
}

function STAGELANDED:New()
  -- Arrange meta tables
  local Child = BASE:Inherit( self, STAGE:New() )
  return Child
end

function STAGELANDED:Execute( Mission, Client, Task )
	Client:Message( 'Co-Pilot: We have landed within the landing zone. Use the radio menu (F10) to ' .. Task.TEXT[1]  .. ' the ' .. _TransportTypeText[Task.CargoType] .. '.', self.MSG.TIME,  Mission.Name .. "/Stage" )
	if not self.MenusAdded then
		Task:RemoveCargoMenus( Client )
		Task:AddCargoMenus( Client, Mission._Cargos, 1000 )
	end
end



function STAGELANDED:Validate( Mission, Client, Task )

  if  _TransportValidateUnitInZone( Client:ClientGroup():getUnits()[1], trigger.misc.getZone(Task.LandingZoneName) ) then
  else
    return -2
  end
  
  if not Client:ClientGroup():getUnits()[1]:inAir() then
  else
    return -1
  end
  
  if  Task.ExecuteStage == _TransportExecuteStage.EXECUTING then
  else
    return 0
  end
  
  return 1
end

STAGEUNLOAD = {
  ClassName = "STAGEUNLOAD",
  MSG = { ID = "Unload", TIME = 10 },
  Name = "Unload"
}

function STAGEUNLOAD:New()
  -- Arrange meta tables
  local Child = BASE:Inherit( self, STAGE:New() )
  return Child
end

function STAGEUNLOAD:Execute( Mission, Client, Task )
  Client:Message( 'Co-Pilot: The ' .. _TransportTypeText[Task.CargoType] .. ' are being ' .. Task.TEXT[2] .. ' within the landing zone. Wait until the helicopter is ' .. Task.TEXT[3] .. '.', self.MSG.TIME,  Mission.Name .. "/Stage" )
  Task:RemoveCargoMenus( Client )
end

function STAGEUNLOAD:Executing( Mission, Client, Task )
	env.info( 'STAGEUNLOAD:Executing()' )
	env.info( 'STAGEUNLOAD:Executing() Task.CargoName = ' .. Task.CargoName ) 
	local Cargo = Client:RemoveCargo( Task.CargoName )
	if Cargo then
		env.info( 'STAGEUNLOAD:Executing() Cargo.CargoName = ' .. Cargo.CargoName ) 
		env.info( 'STAGEUNLOAD:Executing() Cargo.CargoGroupName = ' .. Cargo.CargoGroupName ) 
		env.info( 'STAGEUNLOAD:Executing() Mission._Cargos[Cargo.CargoName].CargoGroupTemplate = ' .. Mission._Cargos[Cargo.CargoName].CargoGroupTemplate ) 
		
		if Cargo.CargoType == _TransportType.INFANTRY then
			if Cargo.CargoName then
				if Task.TargetZoneName then
				  SpawnInfantry ( Client:ClientGroup(), 
						  Mission._Cargos[Cargo.CargoName].CargoGroupTemplate, 
						  Task.TargetZoneName, 
						  0, 
						  0,
						  Mission._Cargos[Cargo.CargoName].CargoGroupName
				  )
				else
				  SpawnInfantry ( Client:ClientGroup(), 
						  Mission._Cargos[Cargo.CargoName].CargoGroupTemplate, 
						  Task.LandingZoneName, 
						  0, 
						  0,
						  Mission._Cargos[Cargo.CargoName].CargoGroupName
				  )
				end
			end
		end
		Task.ExecuteStage = _TransportExecuteStage.SUCCESS
		Client:ShowCargo()
	else
		Client:Message( 'Co-Pilot: There was no cargo in our helicopter of name: ' .. Task.CargoName, 20, Mission.Name .. "/StageFailure" )
		Task.ExecuteStage = _TransportExecuteStage.FAILURE
		Task:Fail()
	end
end

function STAGEUNLOAD:Validate( Mission, Client, Task )
	env.info( 'STAGEUNLOAD:Validate()' )
  
  if  _TransportValidateUnitInZone( Client:ClientGroup():getUnits()[1], trigger.misc.getZone(Task.LandingZoneName) ) then
  else
    Task.ExecuteStage = _TransportExecuteStage.FAILED
    Client:Message( 'Co-Pilot: The ' .. _TransportTypeText[Task.CargoType] .. " haven't been successfully " .. Task.TEXT[3] .. '  within the landing zone. Task and mission has failed.', _TransportStageMsgTime.DONE,  Mission.Name .. "/StageFailure" )
    return 1
  end
  
  if not Client:ClientGroup():getUnits()[1]:inAir() then
  else
    Task.ExecuteStage = _TransportExecuteStage.FAILED
    Client:Message( 'Co-Pilot: The ' .. _TransportTypeText[Task.CargoType] .. " haven't been successfully " .. Task.TEXT[3] .. '  within the landing zone. Task and mission has failed.', _TransportStageMsgTime.DONE,  Mission.Name .. "/StageFailure" )
    return 1
  end
  
  if  Task.ExecuteStage == _TransportExecuteStage.SUCCESS then
    Client:Message( 'Co-Pilot: The ' .. _TransportTypeText[Task.CargoType] .. ' have been sucessfully ' .. Task.TEXT[3] .. '  within the landing zone. Task completed.', _TransportStageMsgTime.DONE,  Mission.Name .. "/StageSuccess" )
	Mission._Cargos[Task.CargoName].Status = CARGOSTATUS.UNLOADED
	env.info( 'STAGEUNLOAD:Validate():unloaded' )
    return 1
  end
  
  return 1
end

STAGELOAD = {
  ClassName = "STAGELOAD",
  MSG = { ID = "Load", TIME = 10 },
  Name = "Load"
}

function STAGELOAD:New()
  -- Arrange meta tables
  local Child = BASE:Inherit( self, STAGE:New() )
  return Child
end

function STAGELOAD:Execute( Mission, Client, Task )
	Client:Message( 'Co-Pilot: The ' .. _TransportTypeText[Task.CargoType] .. ' are being ' .. Task.TEXT[2] .. ' within the landing zone. Wait until the helicopter is ' .. Task.TEXT[3] .. '.', _TransportStageMsgTime.EXECUTING,  Mission.Name .. "/Stage" )
	Task:RemoveCargoMenus( Client )
	-- Route the cargo to the Carrier
	if Mission._Cargos[Task.CargoName].CargoType == _TransportType.INFANTRY then
		Task:OnBoardCargo( Client:ClientGroup(), Mission._Cargos )
		Task.ExecuteStage = _TransportExecuteStage.EXECUTING
	else
	   -- Add the group to the internal cargo;
		Client:AddCargo( Task.CargoName, Mission._Cargos[Task.CargoName].CargoGroupName, Mission._Cargos[Task.CargoName].CargoType, Mission._Cargos[Task.CargoName].CargoWeight, Mission._Cargos[Task.CargoName].CargoGroupTemplate )
		Task.ExecuteStage = _TransportExecuteStage.SUCCESS
	end
end

function STAGELOAD:Executing( Mission, Client, Task )

  -- Remove the loaded object from the battle zone.
  
  if _TransportValidateGroupUnitsInRadius( Group.getByName(Mission._Cargos[Task.CargoName].CargoGroupName), Client:ClientGroup(), 50 ) then
    _TransportDestroyGroupUnitsInRadius( Group.getByName(Mission._Cargos[Task.CargoName].CargoGroupName), Client:ClientGroup(), 50 )
	env.info('trying to remove cargo')

    -- Add the group to the internal cargo;
    Client:AddCargo( Task.CargoName, Mission._Cargos[Task.CargoName].CargoGroupName, Mission._Cargos[Task.CargoName].CargoType, Mission._Cargos[Task.CargoName].CargoWeight, Mission._Cargos[Task.CargoName].CargoGroupTemplate )
	
    -- Message to the pilot that cargo has been loaded.
    Client:Message( "Co-Pilot: The cargo " .. Task.CargoName .. " has been loaded in our helicopter.", 20, Mission.Name .. "/StageSuccess" )
    Task.ExecuteStage = _TransportExecuteStage.SUCCESS
    Client:ShowCargo()
  end
  
end

function STAGELOAD:Validate( Mission, Client, Task )
  if  _TransportValidateUnitInZone( Client:ClientGroup():getUnits()[1], trigger.misc.getZone(Task.LandingZoneName) ) then
  else
    Task.ExecuteStage = _TransportExecuteStage.FAILED
    return 1
  end
  
  if not Client:ClientGroup():getUnits()[1]:inAir() then
  else
    return -1
  end
  
  if Task.ExecuteStage == _TransportExecuteStage.SUCCESS then
    Client:Message( 'Co-Pilot: The ' .. _TransportTypeText[Task.CargoType] .. ' have been sucessfully ' .. Task.TEXT[3] .. '  within the landing zone. Task completed.', _TransportStageMsgTime.DONE,  Mission.Name .. "/StageSuccess" )
	Mission._Cargos[Task.CargoName].Status = CARGOSTATUS.LOADED
    return 1
  end
  
  return 0
end

STAGEDONE = {
  ClassName = "STAGEDONE",
  MSG = { ID = "Done", TIME = 10 },
  Name = "Done"
}

function STAGEDONE:New()
  -- Arrange meta tables
  local Child = BASE:Inherit( self, STAGE:New() )
  return Child
end

function STAGEDONE:Execute( Mission, Client, Task )

end

function STAGEDONE:Validate( Mission, Client, Task )

	Task:Done()
  
	return 0
end

STAGEARRIVE = {
  ClassName = "STAGEARRIVE",
  MSG = { ID = "Arrive", TIME = 10 },
  Name = "Arrive"
}

function STAGEARRIVE:New()
  -- Arrange meta tables
  local Child = BASE:Inherit( self, STAGE:New() )
  return Child
end

function STAGEARRIVE:Execute( Mission, Client, Task )
 
  Client:Message( 'Co-Pilot: We have arrived at ' .. Task.LandingZoneName .. ".", self.MSG.TIME,  Mission.Name .. "/Stage" )

  end

function STAGEARRIVE:Validate( Mission, Client, Task )
  
  if  _TransportValidateUnitInZone( Client:ClientGroup():getUnits()[1], trigger.misc.getZone(Task.LandingZoneName) ) then
  else
    return -1
  end
  
  return 1
end

STAGEGROUPSDESTROYED = {
  ClassName = "STAGEGROUPSDESTROYED",
  DestroyGroupSize = -1,
  Frequency = STAGE.FREQUENCY.REPEAT,
  MSG = { ID = "DestroyGroup", TIME = 10 },
  Name = "GroupsDestroyed"
}

function STAGEGROUPSDESTROYED:New()
  -- Arrange meta tables
  local Child = BASE:Inherit( self, STAGE:New() )
  return Child
end

--function STAGEGROUPSDESTROYED:Execute( Mission, Client, Task )
-- 
--	Client:Message( 'Task: Still ' .. DestroyGroupSize .. " of " .. Task.DestroyGroupCount .. " " .. Task.DestroyGroupType .. " to be destroyed!", self.MSG.TIME,  Mission.Name .. "/Stage" )
--
--end

function STAGEGROUPSDESTROYED:Validate( Mission, Client, Task )
 	
	if self.DestroyGroupSize == 0 then
		Client:Message( Mission.MissionCoalition .. ' command: All ' .. Task.DestroyGroupType .. " were destroyed! Congratulations.", self.MSG.TIME,  Mission.Name .. "/StageSuccess" )
		trigger.action.setUserFlag(200, 1)
		Task:Done()
		return 1
	else
		return 0
	end
end

function STAGEGROUPSDESTROYED:Execute( Mission, Client, Task )
	local DestroyGroupSize = 0
	for DestroyGroupID, DestroyGroupName in pairs( Task.DestroyGroupNames ) do
		DestroyGroup = Group.getByName( DestroyGroupName )
		if DestroyGroup then
			local GroupSize = DestroyGroup:getSize()
			DestroyGroupSize = DestroyGroupSize + GroupSize
		end
	end
	
	self.DestroyGroupSize = DestroyGroupSize
	
	Client:Message( Mission.MissionCoalition .. ' command: ' .. self.DestroyGroupSize .. ' units of ' .. Task.DestroyGroupCount .. " " .. Task.DestroyGroupType .. " to be destroyed!", self.MSG.TIME,  Mission.Name .. "/Stage" )
end













--[[
  _TransportStage: Defines the different stages of which of transport missions can be in. This table is internal and is used to control the sequence of messages, actions and flow.
  
  - _TransportStage.START
  - _TransportStage.ROUTE
  - _TransportStage.LAND
  - _TransportStage.EXECUTE
  - _TransportStage.DONE
  - _TransportStage.REMOVE
--]]
_TransportStage = { 
  HOLD = "HOLD",
  START = "START", 
  ROUTE = "ROUTE", 
  LANDING = "LANDING",
  LANDED = "LANDED",
  EXECUTING = "EXECUTING",
  LOAD = "LOAD",
  UNLOAD = "UNLOAD",
  DONE = "DONE", 
  NEXT = "NEXT"
}

_TransportStageMsgTime = { 
  HOLD = 10,
  START = 60, 
  ROUTE = 5, 
  LANDING = 10,
  LANDED = 30,
  EXECUTING = 30,
  LOAD = 30,
  UNLOAD = 30,
  DONE = 30, 
  NEXT = 0
}

_TransportStageTime = { 
  HOLD = 10,
  START = 5, 
  ROUTE = 5, 
  LANDING = 1,
  LANDED = 1,
  EXECUTING = 5,
  LOAD = 5,
  UNLOAD = 5,
  DONE = 1, 
  NEXT = 0
}

_TransportStageAction = { 
  REPEAT = -1,
  NONE = 0,
  ONCE = 1
}
